@@ -5,7 +5,7 @@ Created on Tue Apr 28 11:37:28 2020
 @author: Dripta
 """
 
-from flask import Flask, jsonify, request, render_template, url_for, send_file, send_from_directory, safe_join, abort
+from flask import Flask, jsonify, request, render_template, url_for, send_file, send_from_directory, safe_join, abort, redirect
 from ..covidindia import *
 from flask import Response
 import json
@@ -16,15 +16,63 @@ import os
 from .hideFile import hider, saveFile
 import shutil
 from platform import system
+import pickle
+from flask_cachebuster import CacheBuster
 
 
 file_path = os.path.dirname(os.path.abspath(__file__))
 if system() == 'Windows':
     if os.path.exists(os.path.join(file_path, 'static', 'files')):
         shutil.rmtree(os.path.join(file_path, 'static', 'files'))
+    if not os.path.exists(os.path.join(file_path, 'static', 'server_data')):
+        os.mkdir(os.path.join(file_path, 'static', 'server_data'))
+        print('Gathering required data....')
+        init = initializer(silent=True)
+        with open(os.path.join(file_path, 'static', 'server_data', 'init.pkl'), 'wb') as init_file:
+            pickle.dump(init, init_file)
+        '''init.csv_Confirmed.to_csv(os.path.join(file_path, 'static', 'server_data','Total_Confirmed.csv'),index=False)
+        init.csv_recovered.to_csv(os.path.join(file_path, 'static', 'server_data','Total_Recovered.csv'),index=False)
+        init.csv_Death.to_csv(os.path.join(file_path, 'static', 'server_data','Total_Death.csv'),index=False)
+        init.count_conf.to_csv(os.path.join(file_path, 'static', 'server_data','Confirmed.csv'),index=False)
+        init.count_recover.to_csv(os.path.join(file_path, 'static', 'server_data','Recovered.csv'),index=False)
+        init.count_death.to_csv(os.path.join(file_path, 'static', 'server_data','Death.csv'),index=False)'''
+        filter_data = Data(init)
+        with open(os.path.join(file_path, 'static', 'server_data', 'filter_data.pkl'), 'wb') as filter_file:
+            pickle.dump(filter_data, filter_file)
+        print('Gathering Demographic data....')
+        demo = Demographic_overview(init, silent=True)
+        with open(os.path.join(file_path, 'static', 'server_data', 'demo.pkl'), 'wb') as demo_file:
+            pickle.dump(demo, demo_file)
+        print('Gathering test data.....')
+        tested_df = filter_data.tested_subject_data()
+        tested_df.to_csv(os.path.join(file_path, 'static',
+                                      'server_data', 'tested_data.csv'), index=False)
 else:
     if os.path.exists(os.path.join(file_path, 'static', '.files')):
         shutil.rmtree(os.path.join(file_path, 'static', '.files'))
+    if not os.path.exists(os.path.join(file_path, 'static', 'server_data')):
+        os.mkdir(os.path.join(file_path, 'static', 'server_data'))
+        print('Gathering required data....')
+        init = initializer(silent=True)
+        with open(os.path.join(file_path, 'static', 'server_data', 'init.pkl'), 'wb') as init_file:
+            pickle.dump(init, init_file)
+        '''init.csv_Confirmed.to_csv(os.path.join(file_path, 'static', 'server_data','Total_Confirmed.csv'),index=False)
+        init.csv_recovered.to_csv(os.path.join(file_path, 'static', 'server_data','Total_Recovered.csv'),index=False)
+        init.csv_Death.to_csv(os.path.join(file_path, 'static', 'server_data','Total_Death.csv'),index=False)
+        init.count_conf.to_csv(os.path.join(file_path, 'static', 'server_data','Confirmed.csv'),index=False)
+        init.count_recover.to_csv(os.path.join(file_path, 'static', 'server_data','Recovered.csv'),index=False)
+        init.count_death.to_csv(os.path.join(file_path, 'static', 'server_data','Death.csv'),index=False)'''
+        filter_data = Data(init)
+        with open(os.path.join(file_path, 'static', 'server_data', 'filter_data.pkl'), 'wb') as filter_file:
+            pickle.dump(filter_data, filter_file)
+        print('Gathering Demographic data....')
+        demo = Demographic_overview(init, silent=True)
+        with open(os.path.join(file_path, 'static', 'server_data', 'demo.pkl'), 'wb') as demo_file:
+            pickle.dump(demo, demo_file)
+        print('Gathering test data.....')
+        tested_df = filter_data.tested_subject_data()
+        tested_df.to_csv(os.path.join(file_path, 'static',
+                                      'server_data', 'tested_data.csv'), index=False)
 
 try:
     os.mkdir(os.path.join(file_path, 'static', 'files'))
@@ -38,16 +86,24 @@ except:
     print("Warning:Can't hide files")
 
 
-print('Gathering required data....')
+'''print('Gathering required data....')
 init = initializer(silent=True)
 filter_data = Data(init)
 print('Gathering Demographic data....')
 demo = Demographic_overview(init, silent=True)
 print('Gathering test data.....')
-tested_df = filter_data.tested_subject_data()
+tested_df = filter_data.tested_subject_data()'''
+
+
 print('Building server')
 app = Flask(__name__, static_url_path='/public')
+app.config["ENV"] = 'development'
+app.config['SERVER_DATA'] = os.path.join(file_path, 'static', 'server_data')
+config = {'extensions': ['.js', '.css', '.csv'], 'hash_size': 5}
 
+cache_buster = CacheBuster(config=config)
+
+cache_buster.init_app(app)
 
 if system() == 'Windows':
     app.config["CLIENT_DATA"] = os.path.join(file_path, 'static', 'files')
@@ -55,6 +111,18 @@ else:
     app.config["CLIENT_DATA"] = os.path.join(file_path, 'static', '.files')
 
 
+tested_df = pd.read_csv(os.path.join(
+    file_path, 'static', 'server_data', 'tested_data.csv'))
+with open(os.path.join(file_path, 'static', 'server_data', 'demo.pkl'), 'rb') as file:
+    demo = pickle.load(file)
+
+with open(os.path.join(file_path, 'static', 'server_data', 'init.pkl'), 'rb') as file:
+    init = pickle.load(file)
+
+with open(os.path.join(file_path, 'static', 'server_data', 'filter_data.pkl'), 'rb') as file:
+    filter_data = pickle.load(file)
+
+
 def test_df(dataset):
     state_list = np.unique(dataset['state'])
     test_table = pd.DataFrame()
@@ -118,17 +186,24 @@ class covserver():
                                    high_occur_death=max(spike_df_death), high_occur_death_date=spike_df_death[spike_df_death == max(spike_df_death)].index[0])
 
         @app.route('/data', methods=['GET', 'POST'])
-        def Data():
+        def data():
             if request.method == 'GET':
-                for i in ['Confirmed', 'Recovered', 'Death']:
+                '''for i in ['Confirmed','Recovered','Death']:
                     df = init.show_data(of=i)
-                    saveFile(df, i)
+                    saveFile(df, i)'''
                 return render_template('index_home.html')
             elif request.method == 'POST':
                 value = request.form['result_data']
-                df = init.show_data(of=value.split(' ')[1])
-                #saveFile(df, value.split(' ')[1])
-                df = df.to_json(orient='records')
+                daily = request.form['daily_data']
+                if daily == 'No':
+                    df = init.show_data(of=value.split(' ')[1])
+                    saveFile(df, f'{value}-{daily}')
+                    df = df.to_json(orient='records')
+                else:
+                    df = init.show_data(of=value.split(' ')[1], daily=True)
+                    saveFile(df, f'{value}-{daily}')
+                    df = df.to_json(orient='records')
+
                 return df
 
         @app.route('/State', methods=['GET', 'POST'])
@@ -138,18 +213,16 @@ class covserver():
             elif request.method == 'POST':
                 state_name = request.form['state_data'].replace('"', '')
                 daily = request.form['daily_data'].replace('"', '')
-                try:
-                    condition = request.form['condition_data'].replace('"', '')
-                except:
-                    pass
                 if state_name == 'All':
                     if daily == 'Yes':
                         df = filter_data.get_dataset_state(
                             state='Whole', daily=True)
-                        return df[condition].to_json(orient='records')
+                        saveFile(df, f'{state_name}-{daily}')
+                        return df.to_json(orient='records')
                     else:
                         df = filter_data.get_dataset_state(
                             state='Whole', daily=False)
+                        saveFile(df, f'{state_name}-{daily}')
                         df = df.to_json(orient='records')
                         return df
                 else:
@@ -158,12 +231,14 @@ class covserver():
                             state=state_name, daily=True).reset_index()
                         df.columns = ['Date', 'Confirmed',
                                       'Recovered', 'Death']
+                        saveFile(df, f'{state_name}-{daily}')
                         df = df.to_json(orient='records')
 
                         return df
                     else:
                         df = filter_data.get_dataset_state(
                             state=state_name, daily=False)
+                        saveFile(df, f'{state_name}-{daily}')
                         df = df.to_json(orient='records')
                         return df
 
@@ -179,7 +254,11 @@ class covserver():
                                          date=date.lower()).reset_index()
                     for i, j in enumerate(df.dateannounced):
                         df.dateannounced[i] = datetime.strftime(j, '%d/%m/%Y')
-
+                    if date != 'All':
+                        tag = date.replace('/', '-')
+                        saveFile(df, f'{place}-({tag})')
+                    else:
+                        saveFile(df, f'{place}-{date}')
                     df = df.to_json(orient='records')
                     return Response(df,  mimetype='application/json')
                 except:
@@ -215,11 +294,14 @@ class covserver():
                             num=num, by=by, kind=kind.lower(), cumulative=False)
                         df = pd.DataFrame(df[state]).reset_index()
                         df.columns = ['Date', f'{state}']
+                        saveFile(
+                            df, f'{kind}-{num}-{by}-Daily-AllDate-{state}')
                         df = df.to_json(orient='records')
                         return Response(df,  mimetype='application/json')
                     else:
                         df = filter_data.rank(
                             num=num, by=by, kind=kind.lower(), cumulative=True)
+                        saveFile(df, f'{kind}-{num}-{by}-Cumulative-AllDate')
                         df = df.to_json(orient='records')
                         return Response(df,  mimetype='application/json')
 
@@ -227,14 +309,26 @@ class covserver():
                     if cumulative == 'False':
                         df = filter_data.rank(
                             num=num, by=by, kind=kind.lower(), cumulative=False, date=date)
+                        saveFile(
+                            df, f'{kind}-{num}-{by}-Daily-({date.replace("/","-")})')
                         df = df.to_json(orient='records')
                         return Response(df,  mimetype='application/json')
                     else:
                         df = filter_data.rank(
                             num=num, by=by, kind=kind.lower(), cumulative=True, date=date)
+                        saveFile(
+                            df, f'{kind}-{num}-{by}-Cumulative-({date.replace("/","-")})')
                         df = df.to_json(orient='records')
                         return Response(df,  mimetype='application/json')
 
+        @app.route('/date', methods=['GET'])
+        def date():
+            with open(os.path.join(file_path, 'static', 'server_data', 'init.pkl'), 'rb') as file:
+                init = pickle.load(file)
+            csv = init.csv_Confirmed
+            lastDate = csv.columns[-1]
+            return lastDate
+
         @app.route('/analysis', methods=['GET', 'POST'])
         def analysis():
             if request.method == 'GET':
@@ -545,6 +639,39 @@ class covserver():
                 dataset = dataset.to_json(orient='records')
                 return Response(dataset,  mimetype='application/json')
 
+        @app.route('/statistics/<gtype>', methods=['GET', 'POST'])
+        def graphtype(gtype):
+            if gtype == 'age_bar_chart':
+                age_data = demo.raw[demo.raw['agebracket']
+                                    != 'Unknown'].reset_index()
+                age_data = age_data.drop(columns=['index'])
+                for i, j in enumerate(age_data['agebracket']):
+                    if '-' in j:
+                        l = j.split('-')
+                        age_data['agebracket'][i] = (float(l[0])+float(l[1]))/2
+                    else:
+                        age_data['agebracket'][i] = float(j)
+                age_bar_dict = {'age': [], 'count': []}
+                for i in np.arange(0, max(age_data['agebracket'])+1, 10):
+                    age_bar_dict['age'].append(f'({i},{i+10})')
+                    age_bar_dict['count'].append(age_data[(age_data['agebracket'] >= i) & (
+                        age_data['agebracket'] < i+10)].shape[0])
+                age_bar = pd.DataFrame(age_bar_dict)
+                age_bar = age_bar.to_json(orient='records')
+                return Response(age_bar,  mimetype='application/json')
+            elif gtype == 'rolling_growth':
+                if request.method == 'POST':
+                    rolling_value = int(request.form['rolling'])
+                    state_data = init.count_conf
+                    state_rolling_data = state_data.iloc[:-1,
+                                                         1:].rolling(rolling_value, axis=1).mean()
+                    state_rolling_data = pd.concat(
+                        [state_data['STATE/UT'][:-1], state_rolling_data], axis=1)
+                    state_rolling_data = state_rolling_data.fillna(0)
+                    state_rolling_data = state_rolling_data.to_json(
+                        orient='records')
+                    return Response(state_rolling_data,  mimetype='application/json')
+
         @app.route('/get-json/<path:filename>', methods=['GET', 'POST'])
         def download_file(filename):
             try:
@@ -562,6 +689,41 @@ class covserver():
             except FileNotFoundError:
                 abort(404)
 
+        @app.route('/refresh/<endpoint>', methods=['GET'])
+        def refresh(endpoint):
+            global init, demo, filter_data
+            init_refresh = initializer(silent=True)
+            if init_refresh.csv_Confirmed.columns[-1] != init.csv_Confirmed.columns[-1]:
+                print('Updating required Data.....', end='\r')
+                with open(os.path.join(file_path, 'static', 'server_data', 'init.pkl'), 'wb') as init_file:
+                    pickle.dump(init_refresh, init_file)
+                filter_data = Data(init_refresh)
+                with open(os.path.join(file_path, 'static', 'server_data', 'filter_data.pkl'), 'wb') as filter_file:
+                    pickle.dump(filter_data, filter_file)
+                print('Updating demographic data....', end='\r')
+                demo = Demographic_overview(init_refresh, silent=True)
+                with open(os.path.join(file_path, 'static', 'server_data', 'demo.pkl'), 'wb') as demo_file:
+                    pickle.dump(demo, demo_file)
+                print('Updating tested data', end='\r')
+                tested_df = filter_data.tested_subject_data()
+                tested_df.to_csv(os.path.join(
+                    file_path, 'static', 'server_data', 'tested_data.csv'), index=False)
+                tested_df = pd.read_csv(os.path.join(
+                    file_path, 'static', 'server_data', 'tested_data.csv'))
+
+                with open(os.path.join(file_path, 'static', 'server_data', 'demo.pkl'), 'rb') as file:
+                    demo = pickle.load(file)
+
+                with open(os.path.join(file_path, 'static', 'server_data', 'init.pkl'), 'rb') as file:
+                    init = pickle.load(file)
+
+                with open(os.path.join(file_path, 'static', 'server_data', 'filter_data.pkl'), 'rb') as file:
+                    filter_data = pickle.load(file)
+
+                return redirect(url_for(f'{endpoint}'))
+            else:
+                return 'Updated'
+
     def run(self, port):
 
         app.run(debug=True, port=port, use_reloader=False)
