@@ -40,7 +40,7 @@ def main():
                         help='process data for selected type', choices=['Total_Confirmed', 'Total_Death',
                                                                         'Total_Recovered'])
     parser.add_argument('--options', type=str,
-                        help='operations user want to do', choices=['show_data',
+                        help='operations user want to do', choices=['show_data', 'demography',
                                                                     'state_dataset', 'data_by_date',
                                                                     'count_by_date', 'rank', 'cumulative_between_date',
                                                                     'count_between_date',
@@ -69,31 +69,64 @@ def main():
             of = args.of
             if '_' in of:
                 of = of.split('_')[1]
+                if args.daily != None:
+                    _daily = True
+                else:
+                    _daily = False
                 if args.head != None:
                     if args.save != None:
-                        init.show_data(of).head(args.head).to_csv(
+                        init.show_data(of, _daily).head(args.head).to_csv(
                             args.save, index=True)
                     else:
-                        print(init.show_data(of).head(args.head))
+                        print(init.show_data(of, _daily).head(args.head))
                 elif args.tail != None:
                     if args.save != None:
-                        init.show_data(of).tail(args.tail).to_csv(
+                        init.show_data(of, _daily).tail(args.tail).to_csv(
                             args.save, index=True)
                     else:
-                        print(init.show_data(of).tail(args.tail))
+                        print(init.show_data(of, _daily).tail(args.tail))
                 elif args.head == None:
                     if args.save != None:
-                        init.show_data(of).to_csv(args.save, index=True)
+                        init.show_data(of, _daily).to_csv(
+                            args.save, index=True)
                     else:
-                        print(init.show_data(of))
+                        print(init.show_data(of, _daily))
                 elif args.tail == None:
                     if args.save != None:
-                        init.show_data(of).to_csv(args.save, index=True)
+                        init.show_data(of, _daily).to_csv(
+                            args.save, index=True)
                     else:
-                        print(init.show_data(of))
+                        print(init.show_data(of, _daily))
             else:
                 print('Invalid Choice')
 
+        elif args.options == 'demography':
+            demo = Demographic_overview(init, silent=True)
+            try:
+                df = demo.demography(place=args.state, date=args.date)
+            except:
+                df = demo.demography(place=args.place, date=args.date)
+            if args.head != None:
+                if args.save != None:
+                    df.head(args.head).to_csv(args.save, index=True)
+                else:
+                    print(df.head(args.head))
+            elif args.tail != None:
+                if args.save != None:
+                    df.tail(args.tail).to_csv(args.save, index=True)
+                else:
+                    print(df.tail(args.tail))
+            elif args.head == None:
+                if args.save != None:
+                    df.to_csv(args.save, index=True)
+                else:
+                    print(df)
+            elif args.tail == None:
+                if args.save != None:
+                    df.to_csv(args.save, index=True)
+                else:
+                    print(df)
+
         elif args.options == 'state_dataset':
             obj2 = Data(init)
             state = args.state
@@ -313,10 +346,6 @@ def main():
                     else:
                         raise Exception('Two Dates must be separated by "-"')
 
-            elif args.graph == 'latitude':
-                obj3.plot_by_latitude(
-                    title='Graph between total confirmed vs latitude')
-
         elif args.options == 'cumulative_between_date':
             obj2 = Data(init)
             if args.of != None:
